module DistributionWrappers
  class Base
    # params: The parameters required by the service (a hash with symbolized keys)
    # tracking: Tracking information to be attached to each result
    # auth: Authentication required by the service
    # baseline: Baseline used by the service (ex. since_id and since_time)

    # name: A descriptive name for the service (based upon the parameters passed)
    # icon: An (optional) image to identify the service by (might be based upon the parameters passed)
    # banner: An (optional) descriptive image provided by the service
    attr_accessor :params

    def initialize(params={}, auth=nil)
      @auth = auth
      @params = params
    end

    protected

    def send_message(recipient)
      @version_contact = PostVersionContactDistro.new
      @version_contact.studio_post_version_id = @params[:version_id]
      @version_contact.distribution_contact_id = recipient['id']
      @version_contact.save
      @params[:version_contact_id] = @version_contact.id
      @message = prepare(recipient['identifier'])
    end

    def redirect_links(doc, add_email=false)
      if ENV['ENVIRONMENT'] == "development"
        base_url = "http://localhost:3000"
        regex = /.*localhost:3000/
      else
        base_url = "http://studio.backstit.ch"
        regex = /.*backstit\.ch/
      end

      links = doc.search('a[href]').reject{ |n| n['href'][regex]}

      email_placeholder = "&email=<<email_placeholder>>" if add_email

      links.each do |link|
        old_link = link.attributes['href'].value
        link.attributes['href'].value = "#{base_url}/posts/#{@params[:post].id}/engagement/redirect_link?url=#{old_link}&contact_id=#{@version_contact.id}&version_id=#{@version_contact.studio_post_version_id}&organization_id=#{@params[:organization].id}#{email_placeholder}"
      end

      doc
    end

    def append_parameters(doc, add_email=false)
      if ENV['ENVIRONMENT'] == 'production'
        regex = /.*backstit\.ch/
      else
        regex = /.*localhost:3000/
      end

      backstitch_links = doc.search('a[href]').select{ |n| n['href'][regex]}

      email_placeholder = "&email=<<email_placeholder>>" if add_email

      backstitch_links.each do |link|
        link.attributes['href'].value = "#{link.attributes['href'].value}&contact_id=#{@version_contact.id}&version_id=#{@version_contact.studio_post_version_id}&organization_id=#{@params[:organization].id}#{email_placeholder}"
      end

      doc
    end
  end
end
