class UserDistro < ActiveRecord::Base  
  self.table_name = "users"
  
  has_many :team_memberships, :foreign_key => "user_id", :class_name => "TeamMemberDistro"
  has_many :teams, :through => :team_memberships
  
  has_many :organizations, :through => :teams

  has_many :organization_admins, :foreign_key => "user_id", :class_name => "OrganizationAdminDistro"
  has_many :managed_organizations, :through => :organization_admins, :source => :organization
  
  has_many :topic_subscriptions, :class_name => 'TopicSubscriptionDistro', :foreign_key => "user_id"
  has_many :topics, :through => :topic_subscriptions
  has_many :owned_topics, :class_name => 'TopicDistro', :as => :owner
  
  def managed_feeds
    self.managed_organizations
        .select("team_feeds.id, feeds.id AS feed_id, teams.id AS team_id, organizations.id AS organization_id, feeds.name, organizations.highlight_color, team_feeds.feed_type")
        .joins({:teams => {:team_feeds => :feed}}).where("team_feeds.feed_type = 'manual'").uniq
  end
  
  def avatar_url 
    avatar = ""
    if self.avatar != nil
      avatar = self.avatar
    else
      hash = Digest::MD5.hexdigest(self.email)
      default = "https://images-backstitch.s3.amazonaws.com/next/logos/backstitch_purple_icon.png"
      avatar = "https://www.gravatar.com/avatar/#{hash}?d=#{default}"
      # "http://unicornify.appspot.com/avatar/#{hash}?s=128"
    end
    avatar
  end
end
